@@ -7,6 +7,7 @@ use serde::{Serialize, Serializer};
 use strum::IntoEnumIterator;
 
 use ruff_linter::FixAvailability;
+use ruff_linter::codes::RuleGroup;
 use ruff_linter::registry::{Linter, Rule, RuleNamespace};
 
 use crate::args::HelpFormat;
@@ -19,9 +20,12 @@ struct Explanation<'a> {
     summary: &'a str,
     message_formats: &'a [&'a str],
     fix: String,
+    fix_availability: FixAvailability,
     #[expect(clippy::struct_field_names)]
     explanation: Option<&'a str>,
     preview: bool,
+    status: RuleGroup,
+    source_location: SourceLocation,
 }
 
 impl<'a> Explanation<'a> {
@@ -36,8 +40,14 @@ impl<'a> Explanation<'a> {
             summary: rule.message_formats()[0],
             message_formats: rule.message_formats(),
             fix,
+            fix_availability: rule.fixable(),
             explanation: rule.explanation(),
             preview: rule.is_preview(),
+            status: rule.group(),
+            source_location: SourceLocation {
+                file: rule.file(),
+                line: rule.line(),
+            },
         }
     }
 }
@@ -122,3 +132,14 @@ pub(crate) fn rules(format: HelpFormat) -> Result<()> {
     }
     Ok(())
 }
+
+/// The location of the rule's implementation in the Ruff source tree, relative to the repository
+/// root.
+///
+/// For most rules this will point to the `#[derive(ViolationMetadata)]` line above the rule's
+/// struct.
+#[derive(Serialize)]
+struct SourceLocation {
+    file: &'static str,
+    line: u32,
+}
