@@ -43,31 +43,10 @@ pub(crate) fn statement(stmt: &Stmt, checker: &mut Checker) {
                     pycodestyle::rules::ambiguous_variable_name(checker, name, name.range());
                 }
             }
-            if checker.is_rule_enabled(Rule::NonlocalWithoutBinding) {
-                pylint::rules::nonlocal_without_binding(checker, nonlocal);
-            }
             if checker.is_rule_enabled(Rule::NonlocalAndGlobal) {
                 pylint::rules::nonlocal_and_global(checker, nonlocal);
             }
         }
-        Stmt::Break(_) => {
-            if checker.is_rule_enabled(Rule::BreakOutsideLoop) {
-                pyflakes::rules::break_outside_loop(
-                    checker,
-                    stmt,
-                    &mut checker.semantic.current_statements().skip(1),
-                );
-            }
-        }
-        Stmt::Continue(_) => {
-            if checker.is_rule_enabled(Rule::ContinueOutsideLoop) {
-                pyflakes::rules::continue_outside_loop(
-                    checker,
-                    stmt,
-                    &mut checker.semantic.current_statements().skip(1),
-                );
-            }
-        }
         Stmt::FunctionDef(
             function_def @ ast::StmtFunctionDef {
                 is_async,
@@ -969,6 +948,9 @@ pub(crate) fn statement(stmt: &Stmt, checker: &mut Checker) {
             if checker.is_rule_enabled(Rule::MisplacedBareRaise) {
                 pylint::rules::misplaced_bare_raise(checker, raise);
             }
+            if checker.is_rule_enabled(Rule::StopIterationReturn) {
+                pylint::rules::stop_iteration_return(checker, raise);
+            }
         }
         Stmt::AugAssign(aug_assign @ ast::StmtAugAssign { target, .. }) => {
             if checker.is_rule_enabled(Rule::GlobalStatement) {
