@@ -1,22 +1,21 @@
-use ruff_diagnostics::{AlwaysFixableViolation, Diagnostic, Fix};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::{self as ast, Expr, Keyword};
 use ruff_text_size::Ranged;
 
 use crate::checkers::ast::Checker;
-
 use crate::rules::flake8_comprehensions::fixes;
+use crate::{AlwaysFixableViolation, Fix};
 
-use super::helpers;
+use crate::rules::flake8_comprehensions::helpers;
 
 /// ## What it does
-/// Checks for unnecessary `list` or `tuple` literals.
+/// Checks for unnecessary list or tuple literals.
 ///
 /// ## Why is this bad?
-/// It's unnecessary to use a list or tuple literal within a call to `dict`.
+/// It's unnecessary to use a list or tuple literal within a call to `dict()`.
 /// It can be rewritten as a dict literal (`{}`).
 ///
-/// ## Examples
+/// ## Example
 /// ```python
 /// dict([(1, 2), (3, 4)])
 /// dict(((1, 2), (3, 4)))
@@ -33,26 +32,27 @@ use super::helpers;
 /// ## Fix safety
 /// This rule's fix is marked as unsafe, as it may occasionally drop comments
 /// when rewriting the call. In most cases, though, comments will be preserved.
-#[violation]
-pub struct UnnecessaryLiteralDict {
-    obj_type: String,
+#[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.61")]
+pub(crate) struct UnnecessaryLiteralDict {
+    obj_type: LiteralKind,
 }
 
 impl AlwaysFixableViolation for UnnecessaryLiteralDict {
     #[derive_message_formats]
     fn message(&self) -> String {
         let UnnecessaryLiteralDict { obj_type } = self;
-        format!("Unnecessary `{obj_type}` literal (rewrite as a `dict` literal)")
+        format!("Unnecessary {obj_type} literal (rewrite as a dict literal)")
     }
 
     fn fix_title(&self) -> String {
-        "Rewrite as a `dict` literal".to_string()
+        "Rewrite as a dict literal".to_string()
     }
 }
 
 /// C406 (`dict([(1, 2)])`)
 pub(crate) fn unnecessary_literal_dict(
-    checker: &mut Checker,
+    checker: &Checker,
     expr: &Expr,
     func: &Expr,
     args: &[Expr],
@@ -63,28 +63,44 @@ pub(crate) fn unnecessary_literal_dict(
     else {
         return;
     };
-    if !checker.semantic().has_builtin_binding("dict") {
-        return;
-    }
     let (kind, elts) = match argument {
-        Expr::Tuple(ast::ExprTuple { elts, .. }) => ("tuple", elts),
-        Expr::List(ast::ExprList { elts, .. }) => ("list", elts),
+        Expr::Tuple(ast::ExprTuple { elts, .. }) => (LiteralKind::Tuple, elts),
+        Expr::List(ast::ExprList { elts, .. }) => (LiteralKind::List, elts),
         _ => return,
     };
     // Accept `dict((1, 2), ...))` `dict([(1, 2), ...])`.
     if !elts
         .iter()
-        .all(|elt| matches!(&elt, Expr::Tuple(ast::ExprTuple { elts, .. }) if elts.len() == 2))
+        .all(|elt| matches!(&elt, Expr::Tuple(tuple) if tuple.len() == 2))
     {
         return;
     }
-    let mut diagnostic = Diagnostic::new(
-        UnnecessaryLiteralDict {
-            obj_type: kind.to_string(),
-        },
-        expr.range(),
-    );
+    if !checker.semantic().has_builtin_binding("dict") {
+        return;
+    }
+    let mut diagnostic =
+        checker.report_diagnostic(UnnecessaryLiteralDict { obj_type: kind }, expr.range());
     diagnostic
         .try_set_fix(|| fixes::fix_unnecessary_literal_dict(expr, checker).map(Fix::unsafe_edit));
-    checker.diagnostics.push(diagnostic);
+}
+
+#[derive(Debug, Copy, Clone, PartialEq, Eq)]
+enum LiteralKind {
+    Tuple,
+    List,
+}
+
+impl LiteralKind {
+    const fn as_str(self) -> &'static str {
+        match self {
+            Self::Tuple => "tuple",
+            Self::List => "list",
+        }
+    }
+}
+
+impl std::fmt::Display for LiteralKind {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.write_str(self.as_str())
+    }
 }
