@@ -49,6 +49,7 @@ use crate::{Edit, Fix, FixAvailability, Violation};
 /// ## References
 /// - [Python documentation: Dictionaries](https://docs.python.org/3/tutorial/datastructures.html#dictionaries)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.30")]
 pub(crate) struct MultiValueRepeatedKeyLiteral {
     name: SourceCodeSnippet,
     existing: SourceCodeSnippet,
@@ -121,6 +122,7 @@ impl Violation for MultiValueRepeatedKeyLiteral {
 /// ## References
 /// - [Python documentation: Dictionaries](https://docs.python.org/3/tutorial/datastructures.html#dictionaries)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.30")]
 pub(crate) struct MultiValueRepeatedKeyVariable {
     name: SourceCodeSnippet,
 }
@@ -176,7 +178,7 @@ pub(crate) fn repeated_keys(checker: &Checker, dict: &ast::ExprDict) {
                     | Expr::EllipsisLiteral(_)
                     | Expr::Tuple(_)
                     | Expr::FString(_) => {
-                        if checker.enabled(Rule::MultiValueRepeatedKeyLiteral) {
+                        if checker.is_rule_enabled(Rule::MultiValueRepeatedKeyLiteral) {
                             let mut diagnostic = checker.report_diagnostic(
                                 MultiValueRepeatedKeyLiteral {
                                     name: SourceCodeSnippet::from_str(checker.locator().slice(key)),
@@ -209,7 +211,7 @@ pub(crate) fn repeated_keys(checker: &Checker, dict: &ast::ExprDict) {
                         }
                     }
                     Expr::Name(_) => {
-                        if checker.enabled(Rule::MultiValueRepeatedKeyVariable) {
+                        if checker.is_rule_enabled(Rule::MultiValueRepeatedKeyVariable) {
                             let mut diagnostic = checker.report_diagnostic(
                                 MultiValueRepeatedKeyVariable {
                                     name: SourceCodeSnippet::from_str(checker.locator().slice(key)),
