@@ -635,93 +635,93 @@ impl fmt::Display for TokenKind {
             TokenKind::TStringEnd => "TStringEnd",
             TokenKind::IpyEscapeCommand => "IPython escape command",
             TokenKind::Comment => "comment",
-            TokenKind::Question => "'?'",
-            TokenKind::Exclamation => "'!'",
-            TokenKind::Lpar => "'('",
-            TokenKind::Rpar => "')'",
-            TokenKind::Lsqb => "'['",
-            TokenKind::Rsqb => "']'",
-            TokenKind::Lbrace => "'{'",
-            TokenKind::Rbrace => "'}'",
-            TokenKind::Equal => "'='",
-            TokenKind::ColonEqual => "':='",
-            TokenKind::Dot => "'.'",
-            TokenKind::Colon => "':'",
-            TokenKind::Semi => "';'",
-            TokenKind::Comma => "','",
-            TokenKind::Rarrow => "'->'",
-            TokenKind::Plus => "'+'",
-            TokenKind::Minus => "'-'",
-            TokenKind::Star => "'*'",
-            TokenKind::DoubleStar => "'**'",
-            TokenKind::Slash => "'/'",
-            TokenKind::DoubleSlash => "'//'",
-            TokenKind::Percent => "'%'",
-            TokenKind::Vbar => "'|'",
-            TokenKind::Amper => "'&'",
-            TokenKind::CircumFlex => "'^'",
-            TokenKind::LeftShift => "'<<'",
-            TokenKind::RightShift => "'>>'",
-            TokenKind::Tilde => "'~'",
-            TokenKind::At => "'@'",
-            TokenKind::Less => "'<'",
-            TokenKind::Greater => "'>'",
-            TokenKind::EqEqual => "'=='",
-            TokenKind::NotEqual => "'!='",
-            TokenKind::LessEqual => "'<='",
-            TokenKind::GreaterEqual => "'>='",
-            TokenKind::PlusEqual => "'+='",
-            TokenKind::MinusEqual => "'-='",
-            TokenKind::StarEqual => "'*='",
-            TokenKind::DoubleStarEqual => "'**='",
-            TokenKind::SlashEqual => "'/='",
-            TokenKind::DoubleSlashEqual => "'//='",
-            TokenKind::PercentEqual => "'%='",
-            TokenKind::VbarEqual => "'|='",
-            TokenKind::AmperEqual => "'&='",
-            TokenKind::CircumflexEqual => "'^='",
-            TokenKind::LeftShiftEqual => "'<<='",
-            TokenKind::RightShiftEqual => "'>>='",
-            TokenKind::AtEqual => "'@='",
-            TokenKind::Ellipsis => "'...'",
-            TokenKind::False => "'False'",
-            TokenKind::None => "'None'",
-            TokenKind::True => "'True'",
-            TokenKind::And => "'and'",
-            TokenKind::As => "'as'",
-            TokenKind::Assert => "'assert'",
-            TokenKind::Async => "'async'",
-            TokenKind::Await => "'await'",
-            TokenKind::Break => "'break'",
-            TokenKind::Class => "'class'",
-            TokenKind::Continue => "'continue'",
-            TokenKind::Def => "'def'",
-            TokenKind::Del => "'del'",
-            TokenKind::Elif => "'elif'",
-            TokenKind::Else => "'else'",
-            TokenKind::Except => "'except'",
-            TokenKind::Finally => "'finally'",
-            TokenKind::For => "'for'",
-            TokenKind::From => "'from'",
-            TokenKind::Global => "'global'",
-            TokenKind::If => "'if'",
-            TokenKind::Import => "'import'",
-            TokenKind::In => "'in'",
-            TokenKind::Is => "'is'",
-            TokenKind::Lambda => "'lambda'",
-            TokenKind::Nonlocal => "'nonlocal'",
-            TokenKind::Not => "'not'",
-            TokenKind::Or => "'or'",
-            TokenKind::Pass => "'pass'",
-            TokenKind::Raise => "'raise'",
-            TokenKind::Return => "'return'",
-            TokenKind::Try => "'try'",
-            TokenKind::While => "'while'",
-            TokenKind::Match => "'match'",
-            TokenKind::Type => "'type'",
-            TokenKind::Case => "'case'",
-            TokenKind::With => "'with'",
-            TokenKind::Yield => "'yield'",
+            TokenKind::Question => "`?`",
+            TokenKind::Exclamation => "`!`",
+            TokenKind::Lpar => "`(`",
+            TokenKind::Rpar => "`)`",
+            TokenKind::Lsqb => "`[`",
+            TokenKind::Rsqb => "`]`",
+            TokenKind::Lbrace => "`{`",
+            TokenKind::Rbrace => "`}`",
+            TokenKind::Equal => "`=`",
+            TokenKind::ColonEqual => "`:=`",
+            TokenKind::Dot => "`.`",
+            TokenKind::Colon => "`:`",
+            TokenKind::Semi => "`;`",
+            TokenKind::Comma => "`,`",
+            TokenKind::Rarrow => "`->`",
+            TokenKind::Plus => "`+`",
+            TokenKind::Minus => "`-`",
+            TokenKind::Star => "`*`",
+            TokenKind::DoubleStar => "`**`",
+            TokenKind::Slash => "`/`",
+            TokenKind::DoubleSlash => "`//`",
+            TokenKind::Percent => "`%`",
+            TokenKind::Vbar => "`|`",
+            TokenKind::Amper => "`&`",
+            TokenKind::CircumFlex => "`^`",
+            TokenKind::LeftShift => "`<<`",
+            TokenKind::RightShift => "`>>`",
+            TokenKind::Tilde => "`~`",
+            TokenKind::At => "`@`",
+            TokenKind::Less => "`<`",
+            TokenKind::Greater => "`>`",
+            TokenKind::EqEqual => "`==`",
+            TokenKind::NotEqual => "`!=`",
+            TokenKind::LessEqual => "`<=`",
+            TokenKind::GreaterEqual => "`>=`",
+            TokenKind::PlusEqual => "`+=`",
+            TokenKind::MinusEqual => "`-=`",
+            TokenKind::StarEqual => "`*=`",
+            TokenKind::DoubleStarEqual => "`**=`",
+            TokenKind::SlashEqual => "`/=`",
+            TokenKind::DoubleSlashEqual => "`//=`",
+            TokenKind::PercentEqual => "`%=`",
+            TokenKind::VbarEqual => "`|=`",
+            TokenKind::AmperEqual => "`&=`",
+            TokenKind::CircumflexEqual => "`^=`",
+            TokenKind::LeftShiftEqual => "`<<=`",
+            TokenKind::RightShiftEqual => "`>>=`",
+            TokenKind::AtEqual => "`@=`",
+            TokenKind::Ellipsis => "`...`",
+            TokenKind::False => "`False`",
+            TokenKind::None => "`None`",
+            TokenKind::True => "`True`",
+            TokenKind::And => "`and`",
+            TokenKind::As => "`as`",
+            TokenKind::Assert => "`assert`",
+            TokenKind::Async => "`async`",
+            TokenKind::Await => "`await`",
+            TokenKind::Break => "`break`",
+            TokenKind::Class => "`class`",
+            TokenKind::Continue => "`continue`",
+            TokenKind::Def => "`def`",
+            TokenKind::Del => "`del`",
+            TokenKind::Elif => "`elif`",
+            TokenKind::Else => "`else`",
+            TokenKind::Except => "`except`",
+            TokenKind::Finally => "`finally`",
+            TokenKind::For => "`for`",
+            TokenKind::From => "`from`",
+            TokenKind::Global => "`global`",
+            TokenKind::If => "`if`",
+            TokenKind::Import => "`import`",
+            TokenKind::In => "`in`",
+            TokenKind::Is => "`is`",
+            TokenKind::Lambda => "`lambda`",
+            TokenKind::Nonlocal => "`nonlocal`",
+            TokenKind::Not => "`not`",
+            TokenKind::Or => "`or`",
+            TokenKind::Pass => "`pass`",
+            TokenKind::Raise => "`raise`",
+            TokenKind::Return => "`return`",
+            TokenKind::Try => "`try`",
+            TokenKind::While => "`while`",
+            TokenKind::Match => "`match`",
+            TokenKind::Type => "`type`",
+            TokenKind::Case => "`case`",
+            TokenKind::With => "`with`",
+            TokenKind::Yield => "`yield`",
         };
         f.write_str(value)
     }
